name = 'badX12'
version = '0.1.0'
author = 'Alberto J. Marin'
author_email = 'alberto@ajmar.in'
homepage = 'https://github.com/git-albertomarin/badX12'
default_user_agent = f'{name}/{version} (+{homepage})'
